@@ -201,6 +201,7 @@ void fdf_read(FILE* in, struct unit_cell *c, struct contents *m,
     if (!fdfstrcasecmp(&ptr,"latticevectors")){
       fdfreadline(buffer,LINE_SIZE,dir,&files);
       vec=malloc(9*sizeof(double));
+      if (!vec) error_exit("malloc error for vectors");
       for(i=0;i<3;i++){
         if (sscanf(buffer,"%lf %lf %lf",vec+3*i,vec+3*i+1,vec+3*i+2)!=3)
           error_exit("Error reading lattice vectors");
@@ -210,6 +211,7 @@ void fdf_read(FILE* in, struct unit_cell *c, struct contents *m,
     }else if (!fdfstrcasecmp(&ptr,"latticeparameters")){
       fdfreadline(buffer,LINE_SIZE,dir,&files);
       abc=malloc(6*sizeof(double));
+      if (!abc) error_exit("malloc error for abc");
       if (sscanf(buffer,"%lf %lf %lf %lf %lf %lf",
                  abc,abc+1,abc+2,abc+3,abc+4,abc+5)!=6)
           error_exit("Error reading lattice parameters");
@@ -220,6 +222,7 @@ void fdf_read(FILE* in, struct unit_cell *c, struct contents *m,
         error_exit("AtomicCoordinatesAndAtomicSpecies preceeds NumberOfAtoms");
       atomspecies=malloc(m->n*sizeof(int));
       atomcoords=malloc(3*m->n*sizeof(double));
+      if ((!atomspecies)||(!atomcoords)) error_exit("malloc error for atoms");
       for(i=0;i<m->n;i++){
         fdfreadline(buffer,LINE_SIZE,dir,&files);
         if (sscanf(buffer,"%lf %lf %lf %d",
@@ -234,6 +237,8 @@ void fdf_read(FILE* in, struct unit_cell *c, struct contents *m,
         error_exit("ChemicalSpeciesLabel preceeds NumberOfSpecies");
       species_to_atno=malloc((nspec+1)*sizeof(int));
       species_to_charge=malloc((nspec+1)*sizeof(double));
+      if ((!species_to_atno)||(!species_to_charge))
+	error_exit("malloc error for species");
       for(i=0;i<nspec;i++){
         fdfreadline(buffer,LINE_SIZE,dir,&files);
         if (sscanf(buffer,"%d %d%n",&j,&k,&n)!=2)
