@@ -324,11 +324,7 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
   /* If we wanted the charge density, now rescue onto a grid */
 
   if ((locpot)||(flags&CHDEN)){
-    if (gptr->next) gptr=gptr->next;
-    gptr->next=malloc(sizeof(struct grid));
-    if (!gptr->next) error_exit("Malloc error for struct grid");
-    gptr->next->next=NULL;
-    gptr->next->data=NULL;
+    gptr=grid_new(gptr);
     
     if(!(gptr->data=malloc(isize*sizeof(double))))
       error_exit("Malloc error in vasp_read for grid data");
@@ -439,11 +435,7 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
         (ffts[1]==fft[1])&&
         (ffts[2]==fft[2])){
       if (debug) fprintf(stderr,"Spin density found\n");
-      if (gptr->next) gptr=gptr->next;
-      gptr->next=malloc(sizeof(struct grid));
-      if (!gptr->next) error_exit("Malloc error for struct grid");
-      gptr->next->next=NULL;
-      gptr->next->data=NULL;
+      gptr=grid_new(gptr);
 
       isize=fft[0]*fft[1]*fft[2];
       if(!(gptr->data=malloc(isize*sizeof(double))))
@@ -687,6 +679,7 @@ void vasp_psi_read(FILE* infile, char *filename, struct unit_cell *c,
     free(c->basis); /* DP version in WAVECAR should be more accurate */
   }
   c->basis=malloc(9*sizeof(double));
+  if (!c->basis) error_exit("Malloc error for basis");
   for(i=0;i<3;i++)
     for(j=0;j<3;j++)
       c->basis[i][j]=basis[i][j];
@@ -1055,6 +1048,7 @@ void vasp_potcar_read(FILE *infile, struct unit_cell *c, struct contents *m,
       if(sscanf(p," ZVAL = %lf",&dtmp)==1){
 	if (i>=nspec){
 	  fprintf(stderr,"Unexpected number of ZVAL entries in POTCAR\n");
+	  free(ch);
 	  return;
 	}
 	ch[i++]=dtmp;
@@ -1062,6 +1056,12 @@ void vasp_potcar_read(FILE *infile, struct unit_cell *c, struct contents *m,
     }
   }
 
+  if (i!=nspec){
+    fprintf(stderr,"Too few ZVAL entries in POTCAR\n");
+    free(ch);
+    return;
+  }
+  
   k=0;
   for(i=0;i<nspec;i++){
     for(j=0;j<nionsp[i];j++){
